/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.zookeeper.server;

import javax.security.auth.Subject;
import javax.security.sasl.SaslException;
import javax.security.sasl.SaslServer;
import java.security.Provider;

import org.apache.zookeeper.Login;
import org.apache.zookeeper.util.SecurityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ZooKeeperSaslServer {
    public static final String LOGIN_CONTEXT_NAME_KEY = "zookeeper.sasl.serverconfig";
    public static final String DEFAULT_LOGIN_CONTEXT_NAME = "Server";

    private static final Logger LOG = LoggerFactory.getLogger(ZooKeeperSaslServer.class);
    private SaslServer saslServer;

    static {
      // trying to get SaslProvider and add it to the list of loaded Security providers
      String saslProviderClassName = System.getProperty("zookeeper.saslprovider");
      if ( saslProviderClassName != null ) {
        try {
          Class<?> clazz = Class.forName(saslProviderClassName);
          java.security.Security.addProvider((Provider) clazz.getDeclaredConstructor().newInstance() );
        } catch (ClassNotFoundException e) {
          LOG.warn("ClassNotFoundException: " + saslProviderClassName, e);
        } catch (InstantiationException e) {
          LOG.warn("InstantiationException: " + saslProviderClassName, e);
        } catch (IllegalAccessException e) {
          LOG.warn("IllegalAccessException: " + saslProviderClassName, e);
        } catch(NoSuchMethodException e) {
          LOG.warn("NoSuchMethodException: " + saslProviderClassName, e);
        } catch(SecurityException e) {
          LOG.warn("SecurityException: " + saslProviderClassName, e);
        } catch(java.lang.reflect.InvocationTargetException e) {
          LOG.warn("InvocationTargetException: " + saslProviderClassName, e);
        }
      }
    }

    ZooKeeperSaslServer(final Login login) {
        saslServer = createSaslServer(login);
    }

    private SaslServer createSaslServer(final Login login) {
        synchronized (login) {
            Subject subject = login.getSubject();
            String mechOpt = System.getProperty("zookeeper.authMech");
            LOG.debug ("ZooKeeperSaslServer::createSaslServer, creating SASL server with mechOpt " + mechOpt);
            return SecurityUtils.createSaslServer(subject, "zookeeper",
                    "zk-sasl-md5", login.callbackHandler, LOG, mechOpt);
        }
    }

    public byte[] evaluateResponse(byte[] response) throws SaslException {
        return saslServer.evaluateResponse(response);
    }

    public boolean isComplete() {
        return saslServer.isComplete();
    }

    public String getAuthorizationID() {
        return saslServer.getAuthorizationID();
    }

}




